#if defined(_WIN32) || defined(__ANDROID__)
#include "../../Common.hpp"
#include "../SoundData.hpp"
#else
#include "SoundPlayer.hpp"
#endif
#include "Global.hpp"
#include "Sound.hpp"

Sound* Sound::sound;

Sound::Sound(void)
{
	for(int i = 0; i < SOUND_MAX; ++ i)
	{
		sound_handle[i] = 0;
	}
}

Sound::~Sound(void)
{
	for(int i = 0; i < SOUND_MAX; ++ i)
	{
		this->sound_handle[i] = 0;
	}
}

void Sound::Initialize(void)
{
	Sound::sound = new Sound();
#if !defined(_WIN32) && !defined(__ANDROID__)
	SoundPlayer::Initialize(11025);
#endif
}

void Sound::Finalize(void)
{
#if !defined(_WIN32) && !defined(__ANDROID__)
	SoundPlayer::Finalize();
#endif
	if(Sound::sound != NULL)
	{
		delete Sound::sound;
		Sound::sound = NULL;
	}
}

Sound& Sound::GetInstance()
{
	return *Sound::sound;
}

void Sound::Load(void)
{
#if defined(_WIN32) || defined(__ANDROID__)
	Common& common = Common::GetInstance();
	dms::SoundPlayer* sound_player = common.resource->GetSoundPlayer(BGM_PLAYER);
#else
	SoundPlayer& sound_player = SoundPlayer::GetInstance();
	this->sound_handle[SOUND_BGM] = sound_player.Append(0, "TinySTG/bgm.raw", true);
	const SoundData* sound_data = GetSoundData(SOUND_CRUSH);
	this->sound_handle[SOUND_CRUSH] = sound_player.Append(1, sound_data->buffer, sound_data->size, false);
	sound_data = GetSoundData(SOUND_CRUSH_BOSS);
	this->sound_handle[SOUND_CRUSH_BOSS] = sound_player.Append(2, sound_data->buffer, sound_data->size, false);
	sound_data = GetSoundData(SOUND_GUN);
	this->sound_handle[SOUND_GUN] = sound_player.Append(3, sound_data->buffer, sound_data->size, false);
	sound_data = GetSoundData(SOUND_POWER_UP);
	this->sound_handle[SOUND_POWER_UP] = sound_player.Append(4, sound_data->buffer, sound_data->size, false);
	sound_data = GetSoundData(SOUND_BOMB);
	this->sound_handle[SOUND_BOMB] = sound_player.Append(4, sound_data->buffer, sound_data->size, false);
	sound_data = GetSoundData(SOUND_GET);
	this->sound_handle[SOUND_GET] = sound_player.Append(4, sound_data->buffer, sound_data->size, false);
#endif
}

void Sound::Release(void)
{
#if defined(_WIN32) || defined(__ANDROID__)
	Common& common = Common::GetInstance();
	dms::SoundPlayer* sound_player = common.resource->GetSoundPlayer(BGM_PLAYER);
	if(sound_player != NULL)
	{
		sound_player->Stop();
	}
#else
	SoundPlayer& sound_player = SoundPlayer::GetInstance();
	sound_player.StopAll();
	sound_player.Delete(this->sound_handle[SOUND_BGM]);
	sound_player.Delete(this->sound_handle[SOUND_CRUSH]);
	sound_player.Delete(this->sound_handle[SOUND_CRUSH_BOSS]);
	sound_player.Delete(this->sound_handle[SOUND_GUN]);
	sound_player.Delete(this->sound_handle[SOUND_POWER_UP]);
	sound_player.Delete(this->sound_handle[SOUND_BOMB]);
	sound_player.Delete(this->sound_handle[SOUND_GET]);
#endif
}

void Sound::Play(int sound_number)
{
	Global& global = Global::GetInstance();
	if(global.gamedata.sound == false)
	{
		return;
	}
#if defined(_WIN32) || defined(__ANDROID__)
	Common& common = Common::GetInstance();
	dms::SoundPlayer* sound_player;
	switch(sound_number)
	{
	case SOUND_BGM:
		sound_player = common.resource->GetSoundPlayer(BGM_PLAYER);
		sound_player->Play(BGM_MAIN);
		break;
	case SOUND_CRUSH:
		sound_player = common.resource->GetSoundPlayer(SE_PLAYER_CRUSH);
		sound_player->Play(SE_CRUSH_CRUSH);
		break;
	case SOUND_CRUSH_BOSS:
		sound_player = common.resource->GetSoundPlayer(SE_PLAYER_CRUSH_BOSS);
		sound_player->Play(SE_CRUSH_BOSS_CRUSH_BOSS);
		break;
	case SOUND_GUN:
		sound_player = common.resource->GetSoundPlayer(SE_PLAYER_GUN);
		sound_player->SetVolume(0.6f);
		sound_player->Play(SE_GUN_GUN);
		break;
	case SOUND_POWER_UP:
		sound_player = common.resource->GetSoundPlayer(SE_PLAYER_COMMON);
		sound_player->Play(SE_COMMON_POWER_UP);
		break;
	case SOUND_BOMB:
		sound_player = common.resource->GetSoundPlayer(SE_PLAYER_COMMON);
		sound_player->Play(SE_COMMON_BOMB);
		break;
	case SOUND_GET:
		sound_player = common.resource->GetSoundPlayer(SE_PLAYER_COMMON);
		sound_player->Play(SE_COMMON_GET);
		break;
	}
#else
	SoundPlayer& sound_player = SoundPlayer::GetInstance();
	sound_player.Play(this->sound_handle[sound_number]);
#endif
}

void Sound::StopAll(void)
{
#if defined(_WIN32) || defined(__ANDROID__)
	Common& common = Common::GetInstance();
	dms::SoundPlayer* sound_player;
	sound_player = common.resource->GetSoundPlayer(BGM_PLAYER);
	if(sound_player != NULL)
	{
		sound_player->Stop();
	}
	sound_player = common.resource->GetSoundPlayer(SE_PLAYER_CRUSH);
	if(sound_player != NULL)
	{
		sound_player->Stop();
	}
	sound_player = common.resource->GetSoundPlayer(SE_PLAYER_CRUSH_BOSS);
	if(sound_player != NULL)
	{
		sound_player->Stop();
	}
	sound_player = common.resource->GetSoundPlayer(SE_PLAYER_GUN);
	if(sound_player != NULL)
	{
		sound_player->Stop();
	}
	sound_player = common.resource->GetSoundPlayer(SE_PLAYER_COMMON);
	if(sound_player != NULL)
	{
		sound_player->Stop();
	}
#else
	SoundPlayer& sound_player = SoundPlayer::GetInstance();
	sound_player.StopAll();
#endif
}

void Sound::Update(void)
{
#if !defined(_WIN32) && !defined(__ANDROID__)
	SoundPlayer& sound_player = SoundPlayer::GetInstance();
	sound_player.Update();
#endif
}
